{*******************************************}
{   Delphi Lab Tools 1.0                    }
{                                           }
{   Statistics  ToolBox                     }
{   UNIT StatAxisLabelTool                  }
{   Includes :                              }
{            - Weibull, Normal and QQ       }
{              series axis label tool       }
{                                           }
{                                           }
{   By Marjan Slatinek                      }
{   Copyright (c) 1999-2001 Dew Research    }
{   All Rights Reserved                     }
{*******************************************}
{:Intruduces axis tool for probabilities plots.}
unit StatAxisLabelTool;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Chart, TeEngine, TeeProcs, Probabilities, MtxVec, Math387,
  Statistics, TeeTools;


{:Type of probability plot.
  Defines the type of probability plot.
  @enum stNormalSeries Normal probabilities plot.
  @enum stWeibullSeries Weibull probabilities plot.
  @enum stQQSeries Quantile-Quantile plot.
  @cat Probabilities Plots
}
type TProbSeriesType = (stNormalSeries, stWeibullSeries, stQQSeries);

{:Axis tool for probabilities plots.
  Axis tool for probabilities plots. This tool was designed to customize
  axis labels for probabilities plots. It should be used with <See Class="TStatProbSeries">
  class.
  @example In the following example normal probabilities plot is built from "scratch" - without
  adding any series/tool to chart at design time.<p>
  <code>
  Uses MtxVec, MtxVecTee, StatSeries, StatAxisLabelTool,
    Math387;
  var Data, XVec, YVec: TVec;
    X1,Y1,X2,Y2: TSample;
  begin
    Data := TVec.Create;
    XVec := TVec.Create;
    YVec := TVec.Create;
    try
      Chart1.FreeAllSeries(nil);
      Chart1.Tools.Clear;
      Chart1.AddSeries(TStatProbSeies.Create(Chart1));
      with TStatAxisLabelTool(Chart1.Tools.Add(TStatAxisLabelTool.Create(Chart1))) do
      begin
        Series := Chart1.Series[0];
        ProbSeriesType := stNormalSeries;
      end;
      // generate some random values for Data vec
      Data.Size(100);
      Data.RandGauss(0.0,1.0); // standard norm. dist.
      StatNormalPlot(Data,XVec,YVec,MinX,MaxX,MinY,MaxY,false);
      With TStatProbSeries(Chart1.Series[0]) do
      begin
        MinX := X1;
        MinY := Y1;
        MaxX := X2;
        MaxY := Y2;
      end;
      DrawValues(XVec,YVec,Chart1.Series[0]);
    finally
      XVec.Destroy;
      YVec.Destroy;
      Data.Destroy;
    end;
  end;
  </code>
  @cat Probabilities Plots
}
type
  TStatAxisLabelTool=class(TTeeCustomToolAxis)
  private
    FProbSeriesType: TProbSeriesType;
    procedure SetProbSeriesType(const Value: TProbSeriesType);
  protected
    procedure SetParentChart(const Value: TCustomAxisPanel); override;
    class Function GetEditorClass:String; override;
    procedure OnGetLabel( Sender:TChartAxis; Series:TChartSeries;
               ValueIndex:Integer; Var LabelText:String);
  public
    {:@exclude}
    Constructor Create(AOwner:TComponent); override;
    {:@exclude}
    class Function Description:String; override;
  published
    {:Defines type of probability series being used.
      Defines type of probability series being used.
      @example
      <code>
      </code>
    }
    property ProbSeriesType: TProbSeriesType read FProbSeriesType write SetProbSeriesType default stNormalSeries;
  end;


implementation

{ TStatAxisLabelTool }

constructor TStatAxisLabelTool.Create(AOwner: TComponent);
begin
  inherited;
  FProbSeriesType := stNormalSeries;
end;

class function TStatAxisLabelTool.Description: String;
begin
  Result:='Probability label';
end;

class function TStatAxisLabelTool.GetEditorClass: String;
begin
  result:='TStatAxisEditor';
end;

procedure TStatAxisLabelTool.OnGetLabel(Sender: TChartAxis;
  Series: TChartSeries; ValueIndex: Integer; var LabelText: String);
var tmp : Double;
    tmpSt : String;
begin
  if Active and Assigned(Axis) and (Sender=Axis) then
    if not(Axis.Horizontal) and (FProbSeriesType <> stQQSeries) then
    begin
      try
         tmpSt := LabelText;
         While Pos(ThousandSeparator,tmpSt)>0 do Delete(tmpSt,Pos(ThousandSeparator,tmpSt),1);
         tmp:=StrToFloat(tmpSt);
         Case FProbSeriesType of
           stNormalSeries : tmp := NormalCDF(tmp,0,1);
           stWeibullSeries : tmp := 1.0 - 1.0/(Exp(Exp(tmp)));
         end;
         LabelText:=FormatFloat(Sender.AxisValuesFormat,tmp);
      except
         on EConvertError do ;
      end;
    end;
end;

procedure TStatAxisLabelTool.SetParentChart(const Value: TCustomAxisPanel);
begin
  inherited;
  if Assigned(ParentChart) then ParentChart.OnGetAxisLabel:=OnGetLabel;
end;

procedure TStatAxisLabelTool.SetProbSeriesType(const Value: TProbSeriesType);
begin
  FProbSeriesType := Value;
  if Assigned(ParentChart) then ParentChart.Invalidate;
end;

initialization
  RegisterTeeTools([TStatAxisLabelTool]);

finalization
  UnRegisterTeeTools([TStatAxisLabelTool]);

end.
